"""
CommandHistory Tool

Core tool providing the a command history pane interface to the consoles command
history.
"""
#---logging---------------------------------------------------------------------
import logging
log = logging.getLogger('CmdHistory')
#log.setLevel(logging.DEBUG)

#---Imports---------------------------------------------------------------------
import wx
import wx.aui as aui

from ptk_lib.tool_manager import Tool
from ptk_lib.message_bus.mb_node import MBNode

from cmdhist_control import CmdHistControl
import cmdhist_icons

#---Addressed message subjects--------------------------------------------------
#show the command history pane, data=(), result=None
SHOW = 'Show'

#---the tools class-------------------------------------------------------------
class CmdHistoryTool(Tool):
    name = 'CmdHistory'
    descrip = 'Core tool for controlling/accessing the command history'
    version = '11.01'
    author = 'T.Charrett'
    requires = ['Console']           
    core = True            
    icon = cmdhist_icons.cmdhistory32
    
    def __init__(self):
        Tool.__init__(self)
        log.info('Initialising tool')

        #register message handlers
        self.msg_node = MBNode(self.msg_bus, 'CmdHistory')
        self.msg_node.set_handler(SHOW,self.msg_show)

        #create the console frame pane
        self.console = self.app.toolmgr.get_tool('Console')
        self.cmdhist = CmdHistControl(self.console.frame, self)
        pane = aui.AuiPaneInfo()

        #setup how to display this panel
        name='Command History'
        pane.Name(name) #id name
        pane.Caption(name) # caption
        pane.Right() #position
        pane.Layer(1)
        pane.Position(0)
        pane.Row(0)
        pane.CloseButton(True) #close button
        pane.MaximizeButton(True)
        pane.MinimizeButton(True)
        pane.Floatable(True)
        pane.BestSize( (350,400) )
        pane.MinSize( (350,300) )
        pane.DestroyOnClose(False)
        self.console.frame.auimgr.AddPane(self.cmdhist, pane)

        #add a menu item to the tools menu to show and hide our pane
        bmp = cmdhist_icons.cmdhistory16.GetBitmap()
        self.console.add_menu_item('tools', wx.NewId(), 'Command History',
                        'Open the command history pane', self.on_show, bmp)

        log.info('Done Initialising tool')

    def Show(self):
        """
        Show the CommandHistory pane in the console window.
        """
        pane=self.console.frame.auimgr.GetPane('Command History')
        pane.Show()
        self.console.frame.auimgr.Update()

    #---message handlers--------------------------------------------------------
    def msg_show(self, msg):
        self.Show()

    #---event handlers--------------------------------------------------------
    def on_show(self,event):
        """
        wxEvent handler for menu items.
        """
        self.Show()
