"""
Console engine tasks:

Engine tasks for getting autocompletions and call tips from the engine
"""


#---Get possible autocompletions------------------------------------------------
def get_auto_comps(globals, locals,word):
    #find if theword contains '.'
    ndot = word.count('.')
    #if the word contains a '.' then we need a dir listing
    if (ndot>0):
        #split the word up
        parts = word.rsplit('.',1)
        if len(parts)==2:
            nchars = len(parts[1])
        else:
            nchars = 0
        #get the python dir() listing
        try:
            wlist = eval('dir('+parts[0]+')',globals, locals)
        except:
            wlist = None
        if wlist is None:
            wlist=[]
    #otherwise use __builtins__ and objects in the __main__ namespace
    else:
        #get the dir() listings
        dirlist = eval('dir()',globals, locals)
        #get the builtins
        builtinlist = eval("dir(__builtins__)",globals, locals)
        #combine            
        wlist = dirlist+builtinlist
        nchars = len(word)
    #sort these and return
    wlist.sort(lambda x, y: cmp(x.upper(), y.upper()))
    return wlist,nchars


#---get call tip----------------------------------------------------------------
def get_call_tip(globals, locals,name):
    #import modules
    import inspect
    import types
    #get a ref to the object
    try:
        obj = eval(name,globals, locals)
    except:
        obj=None        
    if obj is None:
        return None
    #get the doc string
    doc = inspect.getdoc(obj)
    if doc is None: doc=''
    argspec=''
    calltip=''
    #object is a builtin function
    if inspect.isbuiltin(obj):
        # Builtin functions don't have an argspec that we can get.
        calltip = ''+name 
        if doc !='': calltip=calltip+'\n\n'+ doc
    
    #object is a function
    elif inspect.isfunction(obj):
        args = inspect.getargspec(obj)
        argspec = apply(inspect.formatargspec, args)
        calltip = ''+name 
        if argspec!='': calltip=calltip+argspec            
        if doc !='': calltip=calltip+'\n\n'+ doc
    
    #object is a method
    elif inspect.ismethod(obj):
        args = inspect.getargspec(obj)
        #check if method is bound
        try:
            if obj.im_self is None:
                isbound=False
            else:
                isbound=True
        except AttributeError:
            isbound=False
        #if bound we can drop the 'self' from args
        if len(args[0])!=0:
            if isbound==True and args[0][0]=='self':
                args[0].pop(0)
        argspec = apply(inspect.formatargspec, args)
        calltip = ''+name 
        if argspec !='': calltip=calltip+argspec
        if doc !='': calltip=calltip+'\n\n'+doc

    #object is a class
    elif inspect.isclass(obj):
        #------------------------------------
        def get_constructor(object):
            """
            Return constructor for class object, or None if there isn't one.
            copied from wx.py.introspect
            """
            try:
                return object.__init__.im_func
            except AttributeError:
                for base in object.__bases__:
                    constructor = get_constructor(base)
                    if constructor is not None:
                        return constructor
            return None
        #-----------------------------------
        #if a class then we really want to get the constructor
        try:
            con = get_constructor(obj)
        except:
            con = None
        #get constructor args
        try:
            args = inspect.getargspec(con)
            if args[0][0]=='self':
                args[0].pop(0)
            argspec = apply(inspect.formatargspec, args)
        except:
            argspec = ''
        #get constructor doc
        try:
            condoc = con.__doc__
        except AttributeError:
            condoc = ''
        if condoc is None: condoc=''
        #get constructor name
        try:
            conname = con.__name__
        except AttributeError:
            conname=''
        #construct the tip
        calltip = ''+name
        if doc!='': calltip=calltip+'\n\n'+doc
        if conname!='': calltip=calltip+'\n\n'+name+'.'+conname
        if argspec!='': calltip=calltip+argspec
        if condoc!='': calltip=calltip+'\n\n'+condoc

    #is an instance and callable
    elif isinstance(obj,types.InstanceType) and callable(obj):
        #get call args
        try:
            call = obj.__call__.im_func
            args = inspect.getargspec(call)
            argspec = apply(inspect.formatargspec, args)
        except AttributeError:
            argspec=''
        #get class name
        try:
            cname = obj.__class__.__name__
        except AttributeError:
            cname=''
        #get the doc string of the callable
        try:
            doc = inspect.getdoc(call)
            if doc is None: doc=''
        except:
            doc=''
        #construct the tip
        calltip = ''+name
        if argspec!='': calltip=calltip+argspec
        if cname!='':calltip=calltip+'\n\n'+cname
        if doc!='': calltip=calltip+'\n\n'+doc
    
    #something else?
    else:
        try:
            calltip = obj.__doc__
        except:
            #no calltip
            calltip=''

    #check for empty calltips
    if calltip=='':
        calltip=None
    return calltip

