import wx
from ptk_lib.controls import WWStaticText

#---Engine choice dialog--------------------------------------------------------
class EngineChoiceDialog(wx.Dialog):
    def __init__(self,parent,title,engs=[]):
        wx.Dialog.__init__(self,None,-1,title,
                            style=wx.DEFAULT_DIALOG_STYLE,size=(400,300))

        #store the list of engnames
        self.engines = engs

        #create sizer
        sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)

        #engine name box
        namebox = wx.StaticBox(self, -1, "Engine name:")
        boldfont = namebox.GetFont()
        boldfont.SetWeight(wx.BOLD)
        namebox.SetFont(boldfont)
        namesizer = wx.StaticBoxSizer(namebox, wx.VERTICAL)
        self.oldname='Engine-'+str(len(self.engines)+1)
        self.name = wx.TextCtrl(self, -1, self.oldname, size=(125, -1))
        namesizer.Add(self.name,0,wx.EXPAND|wx.ALL,5)
        sizer.Add(namesizer,0,wx.EXPAND|wx.ALL,5)

        #engine type drop list
        app = wx.GetApp()
        tool = app.toolmgr.get_tool('EngineManager')
        self.engdesc = tool.get_engine_descriptions()
        self.engtypes = tool.get_engine_types()

        #remove the internal option if it is already started
        if 'Internal' in self.engines:
            self.engtypes.pop( self.engtypes.index('Internal') )

        typebox = wx.StaticBox(self, -1, "Engine type:")
        typebox.SetFont(boldfont)
        typesizer = wx.StaticBoxSizer(typebox, wx.VERTICAL)
        self.typech = wx.Choice(self, -1, (100, 50), choices = self.engtypes)
        self.Bind(wx.EVT_CHOICE, self.OnEngType, self.typech)
        self.typech.SetSelection(0)
        typesizer.Add(self.typech,0,wx.EXPAND|wx.ALL,5)
        sizer.Add(typesizer,0,wx.EXPAND|wx.ALL,5)

        #engine type description
        descbox = wx.StaticBox(self, -1, "Engine description:" )
        descbox.SetFont(boldfont)
        descsizer = wx.StaticBoxSizer(descbox, wx.VERTICAL)
        self.descrip = WWStaticText(self, -1, self.engdesc['Internal'])
        descsizer.Add(self.descrip,1,wx.EXPAND|wx.ALL,5)
        sizer.Add(descsizer,1,wx.EXPAND|wx.ALL,5)

        #create static line and OK/Cancel button
        line = wx.StaticLine(self,-1)
        sizer.Add(line,0,wx.EXPAND|wx.LEFT|wx.RIGHT,5)

        ok = wx.Button(self, wx.ID_OK, "OK")
        self.Bind(wx.EVT_BUTTON,self.OnOK,ok)
        cancel = wx.Button(self, wx.ID_CANCEL, "Cancel")
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        hsizer.Add( cancel,0,wx.ALIGN_RIGHT|wx.ALL,5)
        hsizer.Add( ok,0,wx.ALIGN_RIGHT|wx.ALL,5)
        sizer.Add(hsizer,0,wx.ALIGN_RIGHT)

        #make sure name if set correctly
        self._CheckName()

    def _CheckName(self):
        """Update the engine description when the engine type is changed"""
        n = self.typech.GetSelection()
        engtype= self.engtypes[n]
        self.descrip.SetLabel(self.engdesc[engtype])

        #check the name
        if engtype=='Internal': #internal engine can only be called internal
            self.oldname=self.name.GetValue()
            self.name.SetValue('Internal')
            self.name.Disable()
            return
    
        self.name.Enable()
        if self.name.GetValue()=='Internal':
            self.name.SetValue(self.oldname)

    def OnEngType(self,event):
        """Update the engine description when the engine type is changed"""
        self._CheckName()
        
    def OnOK(self,event):
        """Check name against autostarted engine dictionary"""
        engname,engtype = self.GetValue()
        #check name
        for name in self.engines:
            if engname==name:
                msg = 'An engine with that name is already exists, the engine name must be unique.'
                title='Engine name already used!'
                dialogs.Message(msg,title)
                return
        event.Skip()

    def GetValue(self):
        """Returns the engine name and selected type"""
        n=self.typech.GetSelection()
        type=str(self.engtypes[n])
        name=self.name.GetValue()
        return name,type

#---Breakpoint edit dialog------------------------------------------------------
class EngineBreakpointDialog(wx.Dialog):
    def __init__(self,parent,bps=[],title='Edit Breakpoints'):
        wx.Dialog.__init__(self, parent,-1,title,
                            style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER,size=(640,320))
    
        #create sizer
        sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)

        #notebook to hold info about multiple breakpoints?
        self.notebook = wx.Notebook(self,-1)
        self.bppanels = []
        for bpid in bps:
            bppanel = BreakPointPanel(self.notebook,-1, bpid)
            self.notebook.AddPage(bppanel, 'Breakpoint: '+str(bpid)) 
            self.bppanels.append(bppanel)
        sizer.Add(self.notebook,1,wx.EXPAND|wx.ALL,5)

        #create static line and OK/Cancel button
        line = wx.StaticLine(self,-1)
        sizer.Add(line,0,wx.EXPAND|wx.LEFT|wx.RIGHT,5)
        apply_but  = wx.Button(self, wx.ID_OK, "Apply")
        cancel_but = wx.Button(self, wx.ID_CANCEL, "Cancel")
        butsizer = wx.BoxSizer(wx.HORIZONTAL)
        butsizer.Add( cancel_but, 0, wx.ALIGN_RIGHT|wx.ALL,5)
        butsizer.Add( apply_but, 0, wx.ALIGN_RIGHT|wx.ALL,5)
        sizer.Add(butsizer,0, wx.ALL|wx.ALIGN_RIGHT)

        self.Bind(wx.EVT_BUTTON, self.OnApply, apply_but)

    def OnApply(self, event):
        for bppanel in self.bppanels:
            bppanel.Apply()
        event.Skip()

class BreakPointPanel(wx.Panel):
    """Panel to hold/edit breakpoint data"""
    def __init__(self, parent, id, bpid):
        wx.Panel.__init__(self, parent, id)
        #create sizer
        sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)

        #filename
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        label = wx.StaticText( self, -1, 'Filename:', size=(100,-1))
        hsizer.Add(label,0,wx.ALIGN_CENTER_VERTICAL|wx.RIGHT,5)
        self.fname = wx.TextCtrl(self,-1, '', size=(200,-1),style=wx.TE_READONLY|wx.TE_RIGHT)
        self.fname.SetBackgroundColour(label.GetBackgroundColour())
        hsizer.Add(self.fname,1,wx.ALIGN_CENTER_VERTICAL|wx.EXPAND|wx.RIGHT,5)

        label = wx.StaticText( self, -1, 'Line number:', size=(100,-1))
        hsizer.Add(label,0,wx.ALIGN_CENTER_VERTICAL|wx.RIGHT,5)
        self.lineno = wx.TextCtrl(self,-1, '', size=(75,-1),style=wx.TE_READONLY)
        self.lineno.SetBackgroundColour(label.GetBackgroundColour())
        hsizer.Add(self.lineno,0,wx.ALIGN_CENTER_VERTICAL,0)
        sizer.Add(hsizer,0,wx.EXPAND|wx.TOP|wx.LEFT|wx.RIGHT,5)

        #condition tick box and text control
        self.cond_check = wx.CheckBox(self,-1,'Evaluate condition')
        self.cond_check.SetValue(False)
        self.Bind(wx.EVT_CHECKBOX, self.OnCondCheck, self.cond_check)
        sizer.Add(self.cond_check,0,wx.LEFT|wx.RIGHT|wx.TOP,5)
        self.cond = wx.TextCtrl(self,-1, '', size=(300,-1),style=wx.TE_MULTILINE)
        self.cond.Disable()
        sizer.Add(self.cond,1,wx.EXPAND|wx.LEFT|wx.RIGHT|wx.TOP,5)

        #ignore count tick box and value control
        self.ignore_check = wx.CheckBox(self,-1,'Ignore count (breakpoint will only trigger after being encountered this many times)')
        self.ignore_check.SetValue(False)
        self.Bind(wx.EVT_CHECKBOX, self.OnIgnoreCheck, self.ignore_check)
        sizer.Add(self.ignore_check,0,wx.LEFT|wx.RIGHT|wx.TOP,5)
        self.ignore = wx.TextCtrl(self,-1, '0')
        self.ignore.Disable()
        sizer.Add(self.ignore,0,wx.LEFT|wx.RIGHT|wx.TOP,5)

        #trigger count tick box and value control
        self.trigger_check = wx.CheckBox(self,-1,'Trigger count (breakpoint will only trigger this many times)')
        self.trigger_check.SetValue(False)
        self.Bind(wx.EVT_CHECKBOX, self.OnTriggerCheck, self.trigger_check)
        sizer.Add(self.trigger_check,0,wx.LEFT|wx.RIGHT|wx.TOP,5)
        self.trigger = wx.TextCtrl(self,-1, '1')
        self.trigger.Disable()
        sizer.Add(self.trigger,0,wx.ALL,5)

        self.SetBreakpoint(bpid)

    def SetBreakpoint(self, bpid):
        """
        Set the breakpoint which the panel is displaying
        """
        app = wx.GetApp()
        engmgr = app.toolmgr.get_tool('EngineManager')
        bpdata = engmgr.get_breakpoint(bpid)
        if bpdata is None:
            return

        self.fname.SetValue( bpdata['filename'])
        self.lineno.SetValue( str(bpdata['lineno']))

        cond = bpdata['condition']
        if cond is None:
            self.cond_check.SetValue(False)
            self.cond.SetValue('')
            self.cond.Disable()
        else:
            self.cond_check.SetValue(True)
            self.cond.SetValue(cond)
            self.cond.Enable()

        ignore_count = bpdata['ignore_count']
        if ignore_count is None:
            self.ignore_check.SetValue(False)
            self.ignore.SetValue('')
            self.ignore.Disable()
        else:
            self.ignore_check.SetValue(True)
            self.ignore.SetValue(str(ignore_count))
            self.ignore.Enable()

        trigger_count = bpdata['trigger_count']
        if trigger_count is None:
            self.trigger_check.SetValue(False)
            self.trigger.SetValue('')
            self.trigger.Disable()
        else:
            self.trigger_check.SetValue(True)
            self.trigger.SetValue(str(trigger_count))
            self.trigger.Enable()

        self.bpdata = bpdata

    def Apply(self):
        """
        Apply any changes made to the breakpoint (modifies the breakpoint via
        the EngineManager tool)
        """
        changes = {}

        #check condition
        if self.cond_check.GetValue() is True:
            cond = self.cond.GetValue()
        else:
            cond = None
        if cond!=self.bpdata['condition']:
            changes['condition']=cond

        #check ignore count
        if self.ignore_check.GetValue() is True:
            try:
                ignore_count = int(self.ignore.GetValue())
            except:
                ignore_count = self.bpdata['ignore_count']
        else:
            ignore_count = None
        if ignore_count!=self.bpdata['ignore_count']:
            changes['ignore_count'] = ignore_count

        #check trigger count
        if self.trigger_check.GetValue() is True:
            try:
                trigger_count = int(self.trigger.GetValue())
            except:
                trigger_count = self.bpdata['trigger_count']
        else:
            trigger_count = None
        if trigger_count!=self.bpdata['trigger_count']:
            changes['trigger_count'] = trigger_count

        if changes!={}:
            app = wx.GetApp()
            engmgr = app.toolmgr.get_tool('EngineManager')
            engmgr.modify_breakpoint(self.bpdata['id'], **changes)

    def OnCondCheck(self,event):
        self.cond.Enable(event.IsChecked())

    def OnIgnoreCheck(self,event):
        self.ignore.Enable(event.IsChecked())

    def OnTriggerCheck(self,event):
        self.trigger.Enable(event.IsChecked())
