"""
PathManager tools

The path manager toolbar
"""
#---logging---------------------------------------------------------------------
import logging
log = logging.getLogger('PathManager')

#---Imports---------------------------------------------------------------------
import os
import wx
from ptk_lib.icons import common16
import pathman_icons                                #icons
from pathlist import PathList               #the path list control
from pathman_dialog import PathManDialog    #the path selection dialog

#---The Pathmanager toolbar-----------------------------------------------------
class PathManTools(wx.ToolBar):
    def __init__(self,parent, tool):

        wx.ToolBar.__init__(self,parent,-1,wx.DefaultPosition, 
                    wx.DefaultSize,wx.TB_FLAT | wx.TB_NODIVIDER)

        self.tool = tool

        #get a reference to the EngineManager tool
        app = wx.GetApp()
        self.engmgr = app.toolmgr.get_tool('EngineManager')

        #subscribe to engine switched messages
        self.tool.msg_node.subscribe('Engine.Switched',self.msg_eng_switched)

        #set the icon size
        size=wx.Size(16, 16)
        self.SetToolBitmapSize(size)

        #load some icons
        bmp_browse  = common16.folder.GetBitmap() 
        bmp_manager   = pathman_icons.pathmanager16.GetBitmap()

        #Current path list
        self.AddControl(wx.StaticText(self, wx.ID_ANY , 'Current directory:   ',size=(-1,16)))
        self.pathlist = PathList(self, tool)
        self.AddControl(self.pathlist)

        #browser
        self.browseid = wx.NewId()
        self.AddLabelTool( self.browseid, "browse", bmp_browse ,shortHelp='Browse for a directory',
                            longHelp='Browse for a directory')
        self.Bind(wx.EVT_TOOL, self.OnBrowse, id=self.browseid)

        #path dialog
        self.manid = wx.NewId()
        self.AddLabelTool( self.manid, "path dialog", bmp_manager,shortHelp='Open the path manager',
                            longHelp='Open the path manager')
        self.Bind(wx.EVT_TOOL, self.OnShowManager, id=self.manid)

        self.Realize()

    #---messages----------------------------------------------------------------
    def msg_eng_switched(self,msg):
        """Engine.Switched message handler"""
        newname,oldname = msg.get_data()
        #check for no active engine
        if newname is None:
            flag = False
        else:
            flag = True

        #enable/disable toolbar buttons
        self.EnableTool(self.manid , flag)

    #---events------------------------------------------------------------------
    def OnBrowse(self,event):
        """open a directory browser"""
        eng=self.engmgr.get_current_engine()
        if eng is None:
            engcwd = os.getcwd()
        else:
            engcwd = eng.run_task('get_cwd')
        dir = wx.DirSelector(defaultPath=engcwd) #use the engines cwd
        if dir!='':
            self.pathlist.ChangeDir(dir)

    def OnShowManager(self,event):
        """open the path manager dialog"""
        #check if there is an engine
        eng=self.engmgr.get_current_engine()
        if eng is None:
            return
        #show the dialog
        d=PathManDialog(None)
        d.ShowModal()
