"""
The python toolkit taskbar icon
"""
#---Imports---------------------------------------------------------------------
import wx

from ptk_lib.icons import ptk_icons
from ptk_lib import VERSION


#---task bar icon---------------------------------------------------------------
class PTKTaskBarIcon(wx.TaskBarIcon):
    def __init__(self, tool):
        wx.TaskBarIcon.__init__(self)
        try:
            if "wxMSW" in wx.PlatformInfo:
                icon = ptk_icons.ptkicon16.GetIcon()
            elif "wxGTK" in wx.PlatformInfo:
                icon = ptk_icons.ptkicon22.GetIcon()
            elif "wxMac" in wx.PlatformInfo:
                icon = ptk_icons.ptkicon48.GetIcon()
        except:
            icon = ptk_icons.ptkicon16.GetIcon()
        
        self.tool = tool
        self.SetIcon(icon, 'Python Toolkit')

        #bind some events
        self.Bind(wx.EVT_TASKBAR_LEFT_DCLICK, self.OnLeftDClick)
        self.Bind(wx.EVT_TASKBAR_RIGHT_DOWN, self.OnMenu)
        
        #create the menu
        self.menu = wx.Menu()   

        self.menu.Append(wx.ID_ABOUT, 'About...', 'About this program...') 
        self.Bind(wx.EVT_MENU, self.OnAbout, id=wx.ID_ABOUT)

        item = self.menu.Append(wx.ID_PREFERENCES, 'Preferences', 'Edit the program preferences')
        self.Bind(wx.EVT_MENU, self.OnPreferences, id=wx.ID_PREFERENCES)
        self.menu.AppendSeparator()

        #Tools add themselves here...

        self.menu.AppendSeparator()
        self.menu.Append(wx.ID_EXIT, 'Exit','Exit the application')
        self.Bind(wx.EVT_MENU, self.OnExit, id=wx.ID_EXIT)

    def OnMenu(self,event):
        """Open the menu"""
        self.PopupMenu(self.menu)

    def OnAbout(self, event):
        """Open the about dialog"""
        PTKInfoDialog()

    def OnPreferences(self,event):
        """Open the preferences dialog"""
        self.tool.show_settings()

    def OnLeftDClick(self,event):
        """Open the console"""
        self.tool.msg_node.send_msg('Console','Show',())
        
    def OnExit(self,event):
        """Exit the application"""
        wx.GetApp().Exit()

#---Info dialog-----------------------------------------------------------------
def PTKInfoDialog():
    """Open the ptk info dialog"""

    descrip = """An interactive environment for Python."""

    licence = """This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation, either version 3 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful,but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program.  If not, see <http://www.gnu.org/licenses/>.
    """
    global VERSION

    info = wx.AboutDialogInfo()
    info.SetIcon( ptk_icons.about.GetIcon())
    #info.SetIcon( ptk_icons.ptkicon48.GetIcon())
    info.SetName('PythonToolkit (PTK)')
    info.SetVersion(VERSION)
    info.SetDescription(descrip)
    info.SetCopyright('(C) 2009 T Charrett')
    info.SetWebSite('http://pythontoolkit.sourceforge.net')
    info.SetLicence(licence)
    info.AddDeveloper('Tom Charrett')
    wx.AboutBox(info)


