"""
TypeView:
-----
A base class for gui views of a specific type.
 - View class should not use the engine interface once DisbleView is called 
   it may be closed and so no longer exist. 
 - The viewer automatically disbales views when the engine is closed.
 - View should publish an Engine.StateChange message after modifying an object
   to allow other tools to update.
"""
import wx

class TypeView(wx.Panel):
    """ 
    A panel class which displays object infomation this is a dummy which does 
    nothing.
    Arguments are parent/viewer, object name and engine.
    """
    def __init__(self,viewer,oname,eng):
        wx.Panel.__init__(self,viewer)
        self.oname = oname
        self.eng = eng
        self.viewer = viewer
        self.disabled = False

    def RefreshView(self):
        """
        This method is called by the Viewer pane to automatically update the 
        contents.
        """
        pass

    def DisableView(self):
        """Disable the view"""
        self.disabled = True
        self.Disable()

    def EnableView(self,eng):
        """Enable the view"""
        self.eng = eng
        self.disabled = False
        self.Enable()


