"""
Package name: PythonToolkit (PTK) engine/console system.
-------------

Description:
-----------
This contains python modules/packages related to the engine/console interaction.
(requires message_bus package for communications)


Components:
-----------

console         -   Console base class.

Engine          -   Abstract base class.
InternalEngine  -   Used for the internal engine -but exists in the controlling
                    process (e.g GUI or console process).

SocketEngine    -   Abstract base class for engines using socket communications

py_engine        - Process used for external engine using python mainloop.
wx_engine        - Process used for external engine with wx mainloop running.
tk_engine        - Process used for external engine with tk mainloop running.
gtk_engine       - Process used for external engine with gtk mainloop.
qt4_engine       - Process used for external engine with Qt4 mainloop.
pyside_engine    - Process used for external engine with PySide Qt4 mainloop.
eng_compiler    - Compiles user source to code.
eng_debugger    - Debugger class
eng_profiler    - Profiler class

messages        -   defines the engine/console communication message types.
eng_misc        -   helper functions/classes for engine management
eng_tasks       -   defines the engine task interfaces and some common tasks.


Overview:
---------
Engines can exist in the same process as the main application or in a seperate 
process and are instance of the the Engine class.

The engine object is created in the external process (external engines) or in
the application process (for the internal engine). It creates a connection 
(node) to the MessageBus running in the interface process which is used for all 
communications between the engine and other program components such as the 
console.The engine can then be controlled by sending and reciving messages over 
the MessageBus, see eng_message for details.

External Engines can have a gui mainloop running or just be a plain python 
interpreter.

Example communications:
-----------------------
1)Engine starts and connects to MessageBus - the engine node is always named 
'Engine.engname' on the message bus.

2)The controlling program should listens for SYS_NODE_CONNECT messages with 
names Engine.engname and creates a console to control the engine and  display
the output etc. (see console.py)

3)The console sends a SET_CONSOLE message to the engine ('Engine.engname') with the
consoles node name (e.g. 'Console.engname'). The engines welcome message and prompts 
are returned and can be displayed in the console.

4)The user enters a command in the console which is sent to the engine using the
PUSH_COMMAND message subject

5)The engine compiles and runs code:

Any output is sent in a CON_WRITE_STDOUT, CON_WRITE_STDERR message to the 
console.
Any input is requested from the console via CON_READ_STDIN message.
When the code completes a CON_PROMPT message is sent to the console.

"""
