"""
gtk engine

An external engine running the gtk mainloop.

More complex than other engines as gtk acquires the python GIL before
calling a python handler preventing the communications thread from running.

Solution: Use gtk.gdk.threads_init and gobject.idle_add

see:
gobject.idle_add
gtk.gdk.threads_init()
gtk.gdk.threads_enter()
gtk.gdk.threads_leave()
"""
#---Logging---------------------------------------------------------------------
import logging
log = logging.getLogger('gtkEngine')
#log.setLevel(logging.DEBUG)

#-------------------------------------------------------------------------------
import gtk
import gobject

from socket_engine import SocketEngine

#-------------------------------------------------------------------------------
def gtk_yield():
    """
    Allowing pending events to be cleared - but do not block
    """
    gtk.main_iteration(False)

class gtkEngine(SocketEngine):
    engtype = 'gtkEngine'

    def __init__(self, engname, userdict={}, 
                    connect=True, address=('localhost',8889), timeout=30):
        
        SocketEngine.__init__(self, engname, userdict, 
                                connect, address, timeout)

    #---Main interface----------------------------------------------------------
    def start_main_loop(self):
        """Wait for user commands to execute"""
        log.debug('starting main loop')
        #release the python Global interpretor lock to allow comms to work!
        gtk.gdk.threads_init()  
        gtk.gdk.threads_enter()
        gtk.main()
        gtk.gdk.threads_leave()

    #---overload base methods---------------------------------------------------
    def run_user_command(self,code):
        """
        Called to run the user code object that has been compiled from a line
        pushed to the engine process. Typically this needs to be run in the main
        thread - not the comms thread which calls this method, so an event of 
        some sort should be raised here to wake the main thread. In this engine
        user code is run in a seperate thread as GTK allows gui calls to be made
        from other threads when the GTK lock is held.
        """ 
        log.debug('in run_user_command')
        gobject.idle_add(self.idle_code, code)
        log.debug('done run_user_command')

    def exit_process(self):
        """
        Things to do when exiting the process
        """
        log.debug('in exit_process')
        #thread_enter/exit causes hang when user types exit() at prompt???
        #gtk.gdk.threads_enter() #in case we're in a thread
        gtk.main_quit()
        #gtk.gdk.threads_leave()

    def get_welcome(self):
        """Return the engines welcome message"""
        welcome = SocketEngine.get_welcome(self) + "\n\nRunning as an external engine process with a GTK mainloop\n"
        return welcome

    #---------------------------------------------------------------------------
    def idle_code(self,code):
        #run code
        gtk.gdk.threads_enter()
        self.run_code(code)
        gtk.gdk.threads_leave()
        log.debug('done idle code')
