"""
Module defining the message subjects used for controlling/interogating engines
and those sent from the engine.
"""
#-------------------------------------------------------------------------------
# Engine control messages
#-------------------------------------------------------------------------------

#Get the engine type, data=(None), result=engtype
ENG_GET_TYPE = 'Eng_Get_Type'

#Set the node name to recieve engine to console messages data=(node_name)
#typically 'Console.engname', returns the engines welcome message string and prompts (ps1, ps2,ps3)
ENG_SET_CONSOLE = 'Eng_Set_Console'      
   
#Push a line of user input to the engine:  data=string, reply=None
ENG_PUSH = 'Eng_Push'       

#Attempt to stop a running user command by raising a keyboard interrupt, reply=None.
ENG_STOP   = 'Eng_Stop'

#execute statement in process, data=string, reply=None
ENG_EXEC_COMMAND = 'Eng_Exec_Command'                       

#evaluate statement in process, data=string, reply=result
ENG_EVAL_COMMAND = 'Eng_Eval_Command'                      

#register a task with the engine process, data=(name,code),reply=None
ENG_REGISTER_TASK = 'Eng_Register_Task'     

#Exectute a task in process, data=code, reply=result
ENG_RUN_TASK = 'Eng_Run_Task'               

#Add a function to the engines builtin module, data=marshalled code, reply=result
ENG_ADD_BUILTIN = 'Eng_Add_Builtin'   

#Enable or disable the future flags from the process compiler, 
#data=(flag, set=True/False)
ENG_FUTURE_FLAG = 'Eng_Future_Flag'  

#Get the engine state reply= (busy, debug,profile, paused)
ENG_GET_STATE = 'Eng_Get_State'  

#Get a list of the registered engine task names, reply= (taskname1, taskname2, ...)
ENG_GET_TASKS = 'Eng_Get_Tasks'  

#-------------------------------------------------------------------------------
#Engine published messages
#-------------------------------------------------------------------------------
#Engine is executing a user command, data=engname, debug, profile
ENGINE_BUSY    = 'Engine.Busy' 

#Engine has finished exectuing a user command, data=engname, debug, profile
ENGINE_DONE    = 'Engine.Done'

# Published when the state of objects in the engine may have changed
# data=(engname, busy, debug, profile)
ENGINE_STATE_CHANGE = 'Engine.StateChange'


#-------------------------------------------------------------------------------
#Console control messages
#-------------------------------------------------------------------------------

#Prompt for next input, data=more (True/False/None=disable prompt), reply=None
CON_PROMPT = 'Con_Prompt'                   

#Read from stdin, data=more(True/False/None=disable prompt), reply=None
CON_PROMPT_STDIN = 'Con_Prompt_StdIn'           

#Prompt for debugger commands data=more (True/False/None=disable prompt), reply=None
CON_PROMPT_DEBUG = 'Con_Prompt_Debug'

#Write to stdout, data=string, reply=None
CON_WRITE_STDOUT = 'Con_Write_StdOut'       

#Write to stdout, data=string, reply=None
CON_WRITE_STDERR = 'Con_Write_StdErr'    

#Write a debugger message, data=string, reply=None
CON_WRITE_DEBUG = 'Con_Write_Debug'

#clear the console, data=None, reply=None - can also be sent by other components
CON_CLEAR = 'Con_Clear'   

#add source to the console and push it to the engine. data=source string.
CON_EXEC_SOURCE = 'Con_Exec_Source'

#-------------------------------------------------------------------------------
#Engine debugger control messages
#-------------------------------------------------------------------------------
#Toggle debug/traceback mode, data=enable (True/False), reply=state (True/False)
ENG_DEBUG_TOGGLE = 'Eng_Debug_Toggle' 

#Pause running user command, reply=sucess (True/False)
ENG_DEBUG_PAUSE  = 'Eng_Debug_Pause'

#Resume running user command, reply=sucess (True/False)
ENG_DEBUG_RESUME = 'Eng_Debug_Resume'         

#End debugging and finish running the user command, reply=sucess (True/False)
ENG_DEBUG_END = 'Eng_Debug_End' 

#Step to next command in debugger, reply=sucess (True/False)
ENG_DEBUG_STEP   = 'Eng_Debug_Step'         

#Step-into scope (only when debugger is on the 'call'), reply=sucess (True/False) 
ENG_DEBUG_STEP_INTO = 'Eng_Debug_Step_Into' 

#Step-out of scope (no traceback until scope finishes, reply=sucess (True/False)
ENG_DEBUG_STEP_OUT = 'Eng_Debug_Step_Out'    

#Set the active scope for interogation, data=level, reply=None
ENG_DEBUG_SET_SCOPE = 'Eng_Debug_Set_Scope'

#Set a debugger break point,
# data=(bpdata= {id,filename, lineno, condition, ignore_count, trigger_count}
# reply=sucess (True/False)
ENG_DEBUG_SET_BREAKPOINT = 'Eng_Debug_Set_Breakpoint'         

#Clear a debugger breakpoint
# data = id (id=None clears all breakpoints)
# reply=sucess (True/False)
ENG_DEBUG_CLEAR_BREAKPOINT = 'Eng_Debug_Clear_Breakpoint'              

#Change a debugger breakpoint
# data = (id, kwargs=(filename, lineno, condition,ignore_count,trigger_count))
# reply=sucess (True/False)
ENG_DEBUG_MODIFY_BREAKPOINT = 'Eng_Debug_Modify_Breakpoint'

#-------------------------------------------------------------------------------
#debugger published messages
#-------------------------------------------------------------------------------

#published when the debugger mode is enabled, data= (engname, enabled=True/False)
ENGINE_DEBUG_TOGGLED = 'Engine.Debug.Toggled'

#paused code.   data=( engname, paused_at=(scope, filename, lineno),
# scope_list, active_scope, flags=(can_stepin,can_stepout) )
ENGINE_DEBUG_PAUSED = 'Engine.Debug.Paused'

#resumed code. data=(engname,)
ENGINE_DEBUG_RESUMED = 'Engine.Debug.Resumed'

#debugger active scope changed, data=(engname, scopes_list, active_scope)
ENGINE_DEBUG_SCOPECHANGED = 'Engine.Debug.ScopeChanged'

#-------------------------------------------------------------------------------
#Engine profiler control messages
#-------------------------------------------------------------------------------

#Toggle profiler mode, data=enable (True/False), reply=state (True/False)
ENG_PROFILE_TOGGLE = 'Eng_Profile_Toggle'

#-------------------------------------------------------------------------------
#profiler published messages
#-------------------------------------------------------------------------------

#published when the profiler mode is enabled, data= (engname, enabled=True/False)
ENGINE_PROFILE_TOGGLED = 'Engine.Profile.Toggled'

