"""
pyEngine:

A basic python engine with no gui mainloops running
- uses a threading.Event() object to wake mainloop and run user command.
"""
#---Logging---------------------------------------------------------------------
import logging
log = logging.getLogger('pyEngine')

#-------------------------------------------------------------------------------
import threading
from socket_engine import SocketEngine

#---External part---------------------------------------------------------------
class pyEngine(SocketEngine):
    engtype = 'pyEngine'

    def __init__(self, engname, userdict={}, 
                    connect=True, address=('localhost',8889), timeout=30):

        #---setup an event to indicate code to run------------------------------
        self.codeevent = threading.Event() #event to indicate code to run
        self.code = None  #code object to run in mainloop as the user

        SocketEngine.__init__(self, engname, userdict,
                                connect, address, timeout)

    #---Main interface----------------------------------------------------------
    def start_main_loop(self):
        """Wait for user commands to execute"""
        while True:
            #check exit flag
            if self._exiting is True:
                break

            #wait for code event or exit
            self.codeevent.wait()
            
            #check exit again
            if self._exiting is True:
                break

            #run code
            self.run_code(self.code) 
            self.code = None
            self.codeevent.clear()

        log.info('Mainloop ended')

    #---overload base methods---------------------------------------------------
    def run_user_command(self,code):
        """
        Called to run the compiled user command that was pushed to the engine 
        process. Typically this needs to be run in the main
        thread - not the comms thread which calls this method, so an event of 
        some sort should be raised here to wake the main thread.
        """ 
        #have a code object so store it and set event
        self.code=code 
        self.codeevent.set()

    def exit_process(self):
        """
        Things to do when exiting the process
        """
        log.info('Exiting process')
        self.codeevent.set() #trigger the event to wake up the mainloop

    def get_welcome(self):
        """Return the engines welcome message"""
        welcome = SocketEngine.get_welcome(self) + "\n\nRunning as an external engine process\n"
        return welcome

