"""
PySide Qt4 External engine using the pySide bindings.

uses the Qt4 mainloop and signals to run user commands.
"""
#---Logging---------------------------------------------------------------------
import logging
log = logging.getLogger('pysideEngine')
log.setLevel(logging.DEBUG)

#-------------------------------------------------------------------------------
from PySide import QtCore, QtGui
#from PyQt4 import QtCore, QtGui

from socket_engine import SocketEngine

#gui yield = self.app.processEvents()

#---External part---------------------------------------------------------------
SigCode = 'UserCode()'
SigClose = 'UserClose()'

class pysideEngine(SocketEngine):
    engtype = 'pysideEngine'
    def __init__(self,engname, userdict={}, 
                    connect=True, address=('localhost',8889), timeout=30):

        self.app = QtGui.QApplication([])
        SocketEngine.__init__(self, engname, userdict, 
                                connect, address, timeout)

        self.app.setApplicationName('PTK PySide Qt4 external engine')
        self.app.setQuitOnLastWindowClosed(False)
        self.code = None
        self.app.connect(self.app, QtCore.SIGNAL(SigCode), self.on_code)
        self.app.connect(self.app, QtCore.SIGNAL(SigClose), self.on_close)

    #---Main interface----------------------------------------------------------
    def start_main_loop(self):
        """Wait for user commands to execute"""
        self.app.exec_()
        log.info('mainloop ended')

    #---overload base methods---------------------------------------------------
    def run_user_command(self,code):
        """
        Called to run the user code object that has been compiled from a line
        pushed to the engine process. Typically this needs to be run in the main
        thread - not the comms thread which calls this method, so an event of 
        some sort should be raised here to wake the main thread.
        """ 
        log.debug('in run_user_command')
        self.code = code
        self.app.emit(QtCore.SIGNAL(SigCode))
        log.debug('done run_user_command')

    def exit_process(self):
        """
        Things to do when exiting the process
        """
        log.debug('in exit_process')
        self.app.emit(QtCore.SIGNAL(SigClose))

    def get_welcome(self):
        """Return the engines welcome message"""
        welcome = SocketEngine.get_welcome(self) +"\n\nRunning as an external engine process with a PySide qt4 mainloop\n"
        return welcome

    #---qt slot event handler---------------------------------------------------
    def on_code(self):
        #run code
        self.run_code(self.code) 
        self.code = None

    def on_close(self):
        self.app.exit()
