"""
SocketEngine - a base class for an external engines using a subclassed
MBSocketClient for communications.

This is subclassed for the different external engines
"""
#---Logging---------------------------------------------------------------------
import logging
log = logging.getLogger('SocketEngine')
log.setLevel(logging.DEBUG)

#---Import----------------------------------------------------------------------
import sys
from ptk_lib.message_bus.mb_socket_client import MBSocketClient  #message bus node
from engine import Engine               #base class

class SocketEngineNode(MBSocketClient):
    def __init__(self, eng, name, address, timeout):
        self.eng = eng
        MBSocketClient.__init__( self, name, address, timeout)

    def on_close(self):
        #when the socket closes we want to stop any running code and exit the
        #engines mainloop
        log.debug('SocketEngineNode closed normally')
        self.eng._close()

    def on_err_close(self):
        #when the socket closes we want to stop any running code and exit the
        #engines mainloop
        log.info('SocketEngineNode closed unexpectedly')
        self.eng._close()


#---External part---------------------------------------------------------------
class SocketEngine(Engine):
    engtype = 'SocketEngine'
    def __init__(self,engname, userdict={}, 
                 connect=True,address=('localhost',8889),timeout=30):

        self.node = SocketEngineNode(self, 'Engine.'+engname, address, timeout)

        #call base class init
        Engine.__init__(self, engname, userdict, self.node)

        if connect is True:
            self.node.connect()
        else:
            self.node.listen()
