"""
Tkinter engine

uses the Tk mainloop and events to run user commands.
"""
#---Logging---------------------------------------------------------------------
import logging
log = logging.getLogger('tkEngine')
log.setLevel(logging.DEBUG)

#-------------------------------------------------------------------------------
import Tkinter as tk
from socket_engine import SocketEngine
#-------------------------------------------------------------------------------

EvtCode  = "<<ENGINE_CODE>>"
EvtClose = "<<ENGINE_CLOSE>>"

#-------------------------------------------------------------------------------
#GUI yield - self.root.update

class TkEngine(SocketEngine):
    engtype = 'tkEngine'

    def __init__(self, engname, userdict={}, 
                    connect=True, address=('localhost',8889), timeout=30):
        #Tk root object
        self.root = tk.Tk()
        SocketEngine.__init__(self, engname, userdict, 
                                connect, address, timeout)

        #bind events
        self.root.bind(EvtCode, self.on_engine_code)
        self.root.bind( EvtClose, self.on_engine_close)

        #prevent root window from displaying
        self.root.withdraw()

    #---Main interface----------------------------------------------------------
    def start_main_loop(self):
        """Wait for user commands to execute"""
        self.root.mainloop()
        log.info('mainloop ended')

    #---overload base methods---------------------------------------------------
    def run_user_command(self,code):
        """
        Called to run the user code object that has been compiled from a line
        pushed to the engine process. Typically this needs to be run in the main
        thread - not the comms thread which calls this method, so an event of 
        some sort should be raised here to wake the main thread.
        """ 
        self.code = code
        self.root.event_generate(EvtCode,when='tail')

    def exit_process(self):
        """
        Things to do when exiting the process
        """
        log.debug('in exit_process')
        self.root.event_generate(EvtClose,when='tail')

    def get_welcome(self):
        """Return the engines welcome message"""
        welcome = SocketEngine.get_welcome(self) + "\n\nRunning as an external engine process with a Tk mainloop\n"
        return welcome

    #---tk event handlers-------------------------------------------------------
    def on_engine_code(self,event):
        #run code
        self.run_code(self.code) 
        self.code = None

    def on_engine_close(self,event):
        log.debug('in OnEngineClose')
        self.root.quit()
