"""
wx external engine:

uses the wx mainloop and wxevents to run user commands.
"""
#---Logging---------------------------------------------------------------------
import logging
log = logging.getLogger('wxEngine')
log.setLevel(logging.DEBUG)

#-------------------------------------------------------------------------------
import wx
from socket_engine import SocketEngine

#---External part---------------------------------------------------------------
# Main thread is a wx.app mainloop waiting for commands or exit flag
from wx.lib.newevent import NewEvent
EvtEngineCode , EVT_ENGINE_CODE = NewEvent() #wx event to run user code
EvtEngineClose , EVT_ENGINE_CLOSE = NewEvent() #wx event to run user code

class wxEngine(SocketEngine):
    engtype = 'wxEngine'
    def __init__(self,engname, userdict={},
                    connect=True, address=('localhost',8889), timeout=30):
        SocketEngine.__init__(self, engname, userdict, 
                                connect, address, timeout)

        #clear sigint allows us to raise a keyboard interrupt in running code.
        self.app = wx.App(redirect=False,clearSigInt=False)
        self.app.SetAppName('wxEngine')

        #---bind wx_User_Code event---------------------------------------------
        self.app.Bind(EVT_ENGINE_CODE, self.OnEngineCode)
        self.app.Bind(EVT_ENGINE_CLOSE, self.OnEngineClose)

        #create a dummy frame to stop wx mainloop from exiting
        self.dummyf = wx.Frame(None,-1,'Dummy')
        #disable exit from mainloop when no frames exist 
        # - this doesn't seem to work?
        self.app.SetExitOnFrameDelete(False) 

    #---Main interface----------------------------------------------------------
    def start_main_loop(self):
        """Wait for user commands to execute"""
        self.app.MainLoop()
        log.info('Mainloop ended')

    #---overload base methods---------------------------------------------------
    def run_user_command(self,code):
        """
        Called to run the user code object that has been compiled from a line
        pushed to the engine process. Typically this needs to be run in the main
        thread - not the comms thread which calls this method, so an event of 
        some sort should be raised here to wake the main thread.
        """ 
        evt = EvtEngineCode(code=code)
        wx.PostEvent(self.app, evt)

    def exit_process(self):
        """
        Things to do when exiting the process
        """
        log.info('Exiting process')
        evt = EvtEngineClose()
        wx.PostEvent(self.app, evt)

    def get_welcome(self):
        """Return the engines welcome message"""
        welcome = SocketEngine.get_welcome(self) + "\n\nRunning as an external engine process with a wx mainloop\n"
        return welcome

    #---wx code event handler---------------------------------------------------
    def OnEngineCode(self,event):
        #run code
        self.run_code(event.code) 

    def OnEngineClose(self,event):
        self.app.Exit()

