"""
wxPython version of the MessageBus & MBSocketClient

"""
#---logging---------------------------------------------------------------------
import logging
log = logging.getLogger('wxMessageBus')
log.setLevel(logging.DEBUG)

#-------------------------------------------------------------------------------
from message_bus import MessageBusBase
from mb_socket_client import MBSocketClient
import mb_protocol

#wx imports
import wx                       
import wx.lib.newevent

#publish event
EvtMsg_Publish, EVT_MSG_PUBLISH = wx.lib.newevent.NewEvent()

#Send message, the reply is stored internally for the waiting thread to collect
EvtMsg_Send , EVT_MSG_SEND  = wx.lib.newevent.NewEvent() 

class wxMessageBus(MessageBusBase):
    def __init__(self, name='MessageBus', timeout=30):
        MessageBusBase.__init__(self,name,timeout)

        #create a wx event handler to catch messenger events
        self.evt_hndlr = wx.EvtHandler()
        self.evt_hndlr.Bind(EVT_MSG_PUBLISH, self.on_publish)
        self.evt_hndlr.Bind(EVT_MSG_SEND,  self.on_send)

    def _do_thread_send(self,msg):

        evt = EvtMsg_Send(msg=msg)
        #post the event
        wx.PostEvent(self.evt_hndlr, evt)

    def _do_thread_publish(self, msg):
        #create the event
        evt = EvtMsg_Publish(msg=msg)
        #post the event
        wx.PostEvent(self.evt_hndlr, evt)

    def on_publish(self,event):
        """
        wxEvent handler to publish a message
        """
        self._do_publish(event.msg)

    def on_send(self,event):
        """
        wxEvent handler to send a message
        """
        self._do_send(event.msg)


#-------------------------------------------------------------------------------
class wxMBSocketClient(MBSocketClient):
    def __init__(self, name, address=('localhost',6667), timeout=10):
        MBSocketClient.__init__(self,name,address,timeout)

        #create a wx event handler to catch messenger events
        self.evt_hndlr = wx.EvtHandler()
        self.evt_hndlr.Bind(EVT_MSG_PUBLISH, self.on_evt_publish)
        self.evt_hndlr.Bind(EVT_MSG_SEND,  self.on_evt_send)

    def on_msg_in(self, msg):
        """
        Handles incomming (from socket) messages.
        This subclass raise an event and calls _do_send fromt he main thread
        which passes the message to any handlers.
        """
        #an addressed messafe
        if (msg.msgtype in [mb_protocol.MSG_ADD_RESULT, mb_protocol.MSG_ADD_NO_RESULT]):
            evt = EvtMsg_Send(msg=msg)
            #post the event
            wx.PostEvent(self.evt_hndlr, evt)

        #a message to publish on the message bus
        elif (msg.msgtype == mb_protocol.MSG_PUBLISHED):
            #create the event
            evt = EvtMsg_Publish(msg=msg)
            #post the event
            wx.PostEvent(self.evt_hndlr, evt)

    #---wxevent handlers--------------------------------------------------------
    def on_evt_publish(self,event):
        """
        wxEvent handler to publish a message
        """
        self._do_publish(event.msg)

    def on_evt_send(self,event):
        """
        wxEvent handler to send a message
        """
        self._do_send(event.msg)
