"""
Misc functions/classes/globals 

USERDIR/setup_log - to setup the logging and get user path for temp storage.
DictList          - a general purpose list/dict object supporting lookup via 
                    multiple keys.
"""
import webbrowser
import logging
from logging import DEBUG,INFO,WARNING,ERROR
import os

#check for and create a .ptk directory in the user home for options, logs and 
#command history
USERDIR = os.path.expanduser('~')+os.sep+'.ptk'+os.sep
if not os.path.exists(USERDIR):
        os.makedirs(USERDIR)
        if not os.path.exists(USERDIR):
            raise AssertionError('Cannot create .ptk options directory in home folder')

TOOLDIR = USERDIR+'tools'+os.sep
if not os.path.exists(TOOLDIR):
        os.makedirs(TOOLDIR)
        if not os.path.exists(TOOLDIR):
            raise AssertionError('Cannot create .ptk/tools directory in home folder')

#-------------------------------------------------------------------------------
def setup_log(filename, level=logging.INFO):
    """
    level - DEBUG,INFO,WARNING,ERROR
    """
    #setup logger
    logging.basicConfig(level=level,
                    format='%(asctime)s - %(name)-16s - %(levelname)-8s - %(message)s',
                    datefmt='%d/%m/%y %H:%M:%S',
                    filename=filename,
                    filemode='w')

def open_help():
    """
    Open the PTK documentation
    """
    url='http://pythontoolkit.sourceforge.net/'
    webbrowser.open(url)
