#!c:\Python27\python.exe
"""
Main PTK launch file - used for launching the application and engines.
"""
import sys

HELP = """PythonToolkit- an interactive python environment:
Usage:

 PTK --help (-h)  Print this help text.

Start the main application: opening the optional filepaths in the editor:
 PTK
 PTK --file (-f) <filepath>

Start an new python interpreter/engine - requires a running PTK instance
to control it via a socket connection:

 PTK --engine (-e) engname, engtype, connect= 'True'/'False', host, port

Where: 
    engname is the name of the engine (which should be unique to the amongst open engines).
    engtype is one of pyEngine, wxEngine, gtkEngine, qt4Engine, tkEngine, pysideEngine depending upon which
        gui toolkit is to be used interactively. pyEngine is a pure python engine with gui 
        toolkit.
    connect = True or False depending upon whether the engine should connect or listen.
    host,port the socket address to connect to or listen on.

Clear all PTK application settings
 PTK --clear_settings (-c)

"""

def start_app(files=[]):
    """
    Start the application
    """
    #setup logging
    from ptk_lib import misc
    LOGFILE  = misc.USERDIR + 'ptk_debug.log'
    LOGLEVEL = misc.INFO # DEBUG,INFO,WARNING,ERROR
    misc.setup_log(filename=LOGFILE, level=LOGLEVEL)

    #create the application instance
    import ptk_lib.app as app
    _app = app.PTKApp(name='PythonToolkit', files=files)

    #set the path to this file for lauching external engines if PTK is not 
    #installed on a system
    import ptk_lib.engine.eng_misc as eng_misc
    eng_misc.PTK_PATH = __file__

    #start main loop
    _app.MainLoop()

def start_engine(engname,engtype,connect,host,port):
    """
    Start an external engine

    args: engname, engtype, connect= 'True'/'False', host, port

    If True the process will connect to the host,port given.
    If False the process will listen on the port given.
    """
    import ptk_lib.misc as misc
    LOGFILE = misc.USERDIR+engname+'.log'
    LOGLEVEL = misc.INFO            #Set the log level - DEBUG,INFO,WARNING,ERROR
    misc.setup_log(filename=LOGFILE, level=LOGLEVEL)

    del LOGFILE, LOGLEVEL, misc
    #-------------------------------------------------------------------------------
    import logging
    log = logging.getLogger('PTKEngine')
    if   engtype == 'pyEngine':
        from ptk_lib.engine.py_engine import pyEngine as Engine
    elif engtype == 'wxEngine':
        from ptk_lib.engine.wx_engine import wxEngine as Engine
    elif engtype == 'gtkEngine':
        from ptk_lib.engine.gtk_engine import gtkEngine as Engine
    elif engtype == 'qt4Engine':
        from ptk_lib.engine.qt4_engine import qt4Engine as Engine
    elif engtype == 'tkEngine':
        from ptk_lib.engine.tk_engine import TkEngine as Engine
    elif engtype == 'pysideEngine':
        from ptk_lib.engine.pyside_engine import pysideEngine as Engine
    else:
        raise Exception('Unknown engine type: '+str(engtype))

    #create the engine
    log.info('Creating engine process.')
    eng = Engine(engname, {}, connect, address=(host,port), timeout=30)

    #start mainloop
    log.info('Starting mainloop.')
    eng.start_main_loop()

    #shutdown cleanly
    log.info('Exiting.')
    logging.shutdown()

def clear_settings():
    import wx
    #create the application instance
    _app = wx.PySimpleApp()
    msg = 'Clear all PTK settings'
    title = 'Clear PTK Settings'
    dlg = wx.MessageDialog(None, msg,title,wx.OK |wx.CANCEL| wx.ICON_EXCLAMATION)
    val = dlg.ShowModal()
    dlg.Destroy()
    if val==wx.ID_OK:
        from ptk_lib import misc
        cfg = wx.FileConfig(localFilename=misc.USERDIR+'options')
        cfg.DeleteAll()

#Process input args
#print sys.argv
#no argument
if len(sys.argv)<=1:
    start_app()
#help
elif sys.argv[1] in ['--help','-h']:
    print HELP
#start an engine
elif sys.argv[1] in ['--engine','-e']:
    try:
        engname = str(sys.argv[2])
        engtype = str(sys.argv[3])
        connect = bool(sys.argv[4])
        host    = sys.argv[5]
        port    = int(sys.argv[6])
    except:
        print HELP
    start_engine(engname,engtype,connect,host,port)
#arguments are files to edit
elif sys.argv[1] in ['--file', '-f']:
    files = sys.argv[2:]
    start_app(files)
#clear settings
elif sys.argv[1] in ['--clear_settings','-c']:
    clear_settings()
else:
    print HELP

